package models

import (
	"gogs.mikescher.com/BlackForestBytes/goext/rfctime"
)

type CrawlerConfig map[string]string

type CrawlerStatus string

type Crawler struct {
	CrawlerID    CrawlerID               `json:"id"                bson:"_id,omitempty"`
	CreationTime rfctime.RFC3339NanoTime `json:"creationTime"      bson:"creationTime"`

	CrawlerEntityData `json:",inline" bson:",inline"`

	CrawlerProcessData `json:",inline" bson:",inline"`
}

// CrawlerEntityData gets modified by the user (frontend, ...)
type CrawlerEntityData struct {
	Enabled        bool                     `json:"enabled"           bson:"enabled"`
	Name           string                   `json:"name"              bson:"name"`
	Interval       rfctime.SecondsF64       `json:"interval"          bson:"interval"`
	RetryInterval  rfctime.SecondsF64       `json:"retryInterval"     bson:"retryInterval"`
	Timeout        rfctime.SecondsF64       `json:"timeout"           bson:"timeout"`
	MaxRetryCount  int                      `json:"maxRetryCount"     bson:"maxRetryCount"`
	Implementation string                   `json:"implementation"    bson:"implementation"`
	Configuration  CrawlerConfig            `json:"configuration"     bson:"configuration"`
	Deleted        *rfctime.RFC3339NanoTime `json:"deleted"           bson:"deleted"`
}

// CrawlerProcessData gets modified by CrawlerManager
type CrawlerProcessData struct {
	LastRun                   *rfctime.RFC3339NanoTime `json:"lastRun"                   bson:"lastRun"`
	LastRunDuration           *rfctime.SecondsF64      `json:"lastRunDuration"           bson:"lastRunDuration"`
	LastSuccesfullRun         *rfctime.RFC3339NanoTime `json:"lastSuccesfullRun"         bson:"lastSuccesfullRun"`
	LastSuccesfullRunDuration *rfctime.SecondsF64      `json:"lastSuccesfullRunDuration" bson:"lastSuccesfullRunDuration"`
	NextRun                   *rfctime.RFC3339NanoTime `json:"nextRun"                   bson:"nextRun"`
	LastRunSuccess            bool                     `json:"lastRunSuccess"            bson:"lastRunSuccess"`
	RunCount                  int                      `json:"runCount"                  bson:"runCount"`
	SuccesfullRunCount        int                      `json:"succesfullRunCount"        bson:"succesfullRunCount"`
	ConsecutiveErrorCount     int                      `json:"consecutiveErrorCount"     bson:"consecutiveErrorCount"`
	ConsecutiveSuccessCount   int                      `json:"consecutiveSuccessCount"   bson:"consecutiveSuccessCount"`
	LastError                 string                   `json:"lastError"                 bson:"lastError"`
	PersistantData            map[string]string        `json:"persistantData"            bson:"persistantData"`
}
