package models

import (
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	ct "gogs.mikescher.com/BlackForestBytes/goext/cursortoken"
)

type SolseitMappingFilter struct {
	SolseitCommunityID  *int
	SolseitRubricID     *int
	BadennetOrg         *OrganizationID
	BadennetGroup       *GroupID
	BadennetGroupDirect *bool
}

func (f SolseitMappingFilter) FilterQuery() mongo.Pipeline {
	pipeline := mongo.Pipeline{}

	if f.SolseitCommunityID != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"solseitCommunityID": *f.SolseitCommunityID}}})
	}

	if f.SolseitRubricID != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"solseitRubricID": *f.SolseitRubricID}}})
	}

	if f.BadennetOrg != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"badennetOrg": *f.BadennetOrg}}})
	}

	if f.BadennetGroup != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"badennetGroup": *f.BadennetGroup}}})
	}

	if f.BadennetGroupDirect != nil {
		if *f.BadennetGroupDirect {
			pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"badennetGroup": bson.M{"$eq": nil}}}})
		} else {
			pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"badennetGroup": bson.M{"$ne": nil}}}})
		}
	}

	return pipeline
}

func (f SolseitMappingFilter) Pagination() (string, ct.SortDirection, string, ct.SortDirection) {
	return "sort", ct.SortASC, "sort", ct.SortASC
}
